@@ -1,7 +1,7 @@
 #
 #       images.R
 #
-#      $Revision: 1.178 $     $Date: 2023/05/04 00:58:05 $
+#      $Revision: 1.184 $     $Date: 2025/06/30 12:04:38 $
 #
 #      The class "im" of raster images
 #
@@ -109,6 +109,12 @@ im <- function(mat, xcol=seq_len(ncol(mat)), yrow=seq_len(nrow(mat)),
   return(out)
 }
 
+getrasterinfo <- function(x, exclude=character(0)) {
+  wanted <- c("dim", "xrange", "yrange", "xstep", "ystep", "xcol", "yrow", "type", "units")
+  wanted <- setdiff(wanted, exclude)
+  unclass(x)[wanted]
+}
+
 is.im <- function(x) {
   inherits(x,"im")
 }
@@ -1086,10 +1092,6 @@ quantilefun.im <- function(x, ..., type=1) {
   quantilefun(f, type=type)
 }
   
-integral <- function(f, domain=NULL, ...) {
-  UseMethod("integral")
-}
-
 integral.im <- function(f, domain=NULL, weight=NULL, ...) {
   verifyclass(f, "im")
   if(!is.null(weight)) {
@@ -1368,17 +1370,3 @@ fillNA <- function(x, value=0) {
   return(x)
 }
 
-
-## probability integral transformation
-## aka histogram equalisation
-## aka transformation to uniformity
-
-transformquantiles <- function(X, uniform=FALSE, reverse=FALSE, ...) {
-  if(!uniform && !reverse) return(X)
-  o <- order(X[])
-  V <- X
-  n <- length(o)
-  if(uniform) V[][o] <- (seq_len(n) - 0.5)/n
-  if(reverse) V[][o] <- V[][rev(o)]
-  return(V)
-}
