@@ -7,7 +7,7 @@
 ##
 ## plot.solist is defined in plot.solist.R
 ##
-## $Revision: 1.28 $ $Date: 2022/08/27 04:11:07 $
+## $Revision: 1.33 $ $Date: 2025/07/06 04:33:54 $
 
 anylist <- function(...) {
   x <- list(...)
@@ -69,24 +69,27 @@ is.sob <- local({
                       "funxy", "distfun", "nnfun", 
                       "lpp", "linnet", "linfun", "lintess",  
                       "influence.ppm", "leverage.ppm")
-  # Note 'linim' inherits 'im'
-  #      'dfbetas.ppm' inherits 'msr'
-
+  ## Note 'linim' inherits 'im'
+  ##      'dfbetas.ppm' inherits 'msr'
+  ##      diagram objects typically inherit 'ppp'
   is.sob <- function(x) { inherits(x, what=sobjectclasses) }
   is.sob
 })
   
 solist <- function(..., check=TRUE, promote=TRUE, demote=FALSE, .NameBase) {
   stuff <- list(...)
-  if(length(stuff) && !missing(.NameBase) && !any(nzchar(names(stuff))))
-    names(stuff) <- paste(.NameBase, seq_along(stuff))
-  if((check || demote) && !all(sapply(stuff, is.sob))) {
-    if(demote)
-      return(as.anylist(stuff))
-    stop("Some arguments of solist() are not 2D spatial objects")
+  if(length(stuff)) {
+    if(!missing(.NameBase) && !any(nzchar(names(stuff))))
+      names(stuff) <- paste(.NameBase, seq_along(stuff))
+    if(check || promote)
+      stuff <- coerceNAtoObject(stuff)
+    if((check || demote) && !all(sapply(stuff, is.sob))) {
+      if(demote) return(as.anylist(stuff)) else
+      stop("Some arguments of solist() are not 2D spatial objects")
+    }
   }
   class(stuff) <- unique(c("solist", "anylist", "listof", class(stuff)))
-  if(promote) {
+  if(promote && length(stuff)) {
     if(all(sapply(stuff, is.ppp))) {
       class(stuff) <- c("ppplist", class(stuff))
     } else if(all(sapply(stuff, is.im))) {
@@ -98,6 +101,20 @@ solist <- function(..., check=TRUE, promote=TRUE, demote=FALSE, .NameBase) {
   return(stuff)
 }
 
+coerceNAtoObject <- function(x, cls=NULL) {
+  ## coerce vanilla NA entries x[[i]]
+  ## to a spatial NA object, if class is unambiguous
+  if(any(isvna <- sapply(x, identical, y=NA))) {
+    if(is.null(cls)) 
+      cls <- unique(sapply(x[!isvna], classIgnoringNA, first=TRUE))
+    if(length(cls) == 1) {
+      nao <- NAobject(cls)
+      x[isvna] <- rep(list(nao), sum(isvna))
+    }
+  }
+  return(x)
+}
+
 as.solist <- function(x, ...) {
   if(inherits(x, "solist") && length(list(...)) == 0) {
     #' wipe superfluous info
@@ -145,6 +162,15 @@ print.solist <- function (x, ...) {
   return(do.call(solist, y))
 }
   
+"[[<-.solist" <- function(x, i, value) {
+  ## invoke list method
+  y <- NextMethod("[[<-")
+  ## check again
+  return(do.call(solist, y))
+}
+
+is.na.solist <- function(x) { sapply(x, is.NAobject) }
+
 summary.solist <- function(object, ...) {
   x <- lapply(object, summary, ...)
   attr(x, "otype") <-
