@@ -1,7 +1,7 @@
 #
 #	wingeom.R	Various geometrical computations in windows
 #
-#	$Revision: 4.139 $	$Date: 2021/10/03 02:37:31 $
+#	$Revision: 4.146 $	$Date: 2025/06/29 07:20:53 $
 #
 
 volume.owin <- function(x) { area.owin(x) }
@@ -124,7 +124,7 @@ square <- function(r=1, unitname=NULL) {
   } else if(length(r) == 2) {
     stopifnot(r[1L] < r[2L])
   } else stop("argument r must be a single number, or a vector of length 2")
-  owin(r,r, unitname=unitname)
+  owinInternalRect(r,r, unitname=unitname)
 }
 
 # convert polygonal window to mask window 
@@ -330,7 +330,7 @@ intersect.owin <- function(..., fatal=FALSE, p) {
   if(!fatal && (is.null(xr) || is.null(yr)))
     return(emptywindow(A))
   #' non-empty intersection of Frames
-  C <- owin(xr, yr, unitname=uname)
+  C <- owinInternalRect(xr, yr, unitname=uname)
 
   # Determine type of intersection
   Arect <- is.rectangle(A)
@@ -532,7 +532,7 @@ union.owin <- function(..., p) {
 
   ## Create a rectangle to contain the result
   
-  C <- owin(range(A$xrange, B$xrange),
+  C <- owinInternalRect(range(A$xrange, B$xrange),
             range(A$yrange, B$yrange),
             unitname=uname)
 
@@ -710,7 +710,7 @@ trim.mask <- function(M, R, tolerant=TRUE) {
     ## R is a rectangle
 
     ## Ensure R is a subset of bounding rectangle of M
-    R <- owin(intersect.ranges(M$xrange, R$xrange),
+    R <- owinInternalRect(intersect.ranges(M$xrange, R$xrange),
               intersect.ranges(M$yrange, R$yrange))
     
     ## Deal with very thin rectangles
@@ -784,7 +784,7 @@ trim.rectangle <- function(W, xmargin=0, ymargin=xmargin) {
     stop("window is too small to cut off margins of the width specified")
   if(sum(ymargin) > diff(W$yrange))
     stop("window is too small to cut off margins of the height specified")
-  owin(W$xrange + c(1,-1) * xmargin,
+  owinInternalRect(W$xrange + c(1,-1) * xmargin,
        W$yrange + c(1,-1) * ymargin,
        unitname=unitname(W))
 }
@@ -800,7 +800,7 @@ grow.rectangle <- function(W, xmargin=0, ymargin=xmargin, fraction=NULL) {
   ymargin <- ensure2vector(ymargin)
   if(any(xmargin < 0) || any(ymargin < 0))
     stop("values of xmargin, ymargin must be nonnegative")
-  owin(W$xrange + c(-1,1) * xmargin,
+  owinInternalRect(W$xrange + c(-1,1) * xmargin,
        W$yrange + c(-1,1) * ymargin,
        unitname=unitname(W))
 }
@@ -911,14 +911,19 @@ diameter.owin <- function(x) {
   w <- as.owin(x)
   if(is.empty(w))
     return(NULL)
-  vert <- vertices(w)
-  if(length(vert$x) > 3) {
-    # extract convex hull
-    h <- with(vert, chull(x, y))
-    vert <- with(vert, list(x=x[h], y=y[h]))
+  if(w$type == "rectangle") {
+    d <- sqrt(diff(w$xrange)^2+diff(w$yrange)^2)
+  } else {
+    vert <- vertices(w)
+    if(length(vert$x) > 3) {
+      ## extract convex hull
+      h <- with(vert, chull(x, y))
+      vert <- with(vert, list(x=x[h], y=y[h]))
+    }
+    d2 <- pairdist(vert, squared=TRUE)
+    d <- sqrt(max(d2))
   }
-  d <- pairdist(vert, squared=TRUE)
-  return(sqrt(max(d)))
+  return(d)
 }
 
 ##    radius of inscribed circle
@@ -992,13 +997,37 @@ inpoint <- function(W) {
 
 simplify.owin <- function(W, dmin) {
   verifyclass(W, "owin")
-  if(is.empty(W))
+  if(is.empty(W) || is.rectangle(W))
     return(W)
   W <- as.polygonal(W)
   W$bdry <- lapply(W$bdry, simplify.xypolygon, dmin=dmin)
   return(W)
 }
 
+fillholes.owin <- function(W, amin) {
+  verifyclass(W, "owin")
+  switch(W$type,
+         rectangle = { },
+         polygonal = {
+           a <- sapply(W$bdry, Area.xypolygon)
+           if(any(tinyhole <- (a <= 0 & abs(a) < amin)))
+             W$bdry <- W$bdry[!tinyhole]
+         },
+         mask = {
+           co <- connected(complement.owin(W))
+           te <- tess(image=co)
+           a <- tile.areas(te)
+           if(any(tinyhole <- (a < amin))) {
+             lev <- levels(co)
+             for(i in which(tinyhole)) {
+               levi <- lev[i]
+               W$m[(co == levi)$v] <- TRUE
+             }
+           }
+         })
+  return(W)
+}
+
   
 is.convex <- function(x) {
   verifyclass(x, "owin")
@@ -1072,7 +1101,7 @@ discs <- function(centres, radii=marks(centres)/2, ...,
   stopifnot(is.ppp(centres))
   n <- npoints(centres)
   if(n == 0) return(emptywindow(Frame(centres)))
-  check.nvector(radii, npoints(centres), oneok=TRUE)
+  check.nvector(radii, npoints(centres), oneok=TRUE, vname="radii")
   stopifnot(all(radii > 0))
   
   if(sameradius <- (length(radii) == 1)) 
